@@ -1,5 +1,5 @@
 // Source file for evolvotron
-// Copyright (C) 2002,2003,2004 Tim Day
+// Copyright (C) 2007 Tim Day
 /*
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,44 +21,28 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   \todo Eliminate need to include function.h (and instantiate lots of stuff) by moving more into function_node.h/.cpp
 */
 
-#include <time.h>
-#include <qapplication.h>
-#include <qcursor.h>
-#include <qdatetime.h>
+#include "libevolvotron_precompiled.h"
+
+#include "evolvotron_main.h"
+
 #include <qmessagebox.h>
 #include <qtooltip.h>
 #include <qregexp.h>
 
 #include "args.h"
-#include "evolvotron_main.h"
-#include "xyz.h"
-#include "function_node.h"
-#include "function_pre_transform.h"
-#include "function_post_transform.h"
-
 #include "dialog_about.h"
 #include "dialog_help.h"
 #include "dialog_mutation_parameters.h"
 #include "dialog_functions.h"
 #include "dialog_favourite.h"
+#include "function_node.h"
+#include "function_post_transform.h"
+#include "function_pre_transform.h"
+#include "function_top.h"
 
 void EvolvotronMain::History::purge()
 {
-  if (_history.size()>0)
-    {
-      // Clean up any images at back end of queue
-      for (
-	   std::vector<std::pair<MutatableImageDisplay*,MutatableImage*> >::iterator it=_history.back().second.begin();
-	   it!=_history.back().second.end();
-	   it++
-	   )
-	{
-	  delete (*it).second;
-	}
-
-      // Get rid of the entry
-      _history.pop_back();
-    }
+  if (_archive.size()>0) _archive.pop_back();
 }
 
 EvolvotronMain::History::History(EvolvotronMain* m)
@@ -69,70 +53,48 @@ EvolvotronMain::History::History(EvolvotronMain* m)
 }
 
 EvolvotronMain::History::~History()
+{}
+
+void EvolvotronMain::History::log_status() const
 {
-  while (_history.size()>0)
-    {
-      purge();
-    }  
+  std::clog << "[History: " << _archive.size() << " records (";
+  for (Archive::const_iterator it=_archive.begin();it!=_archive.end();it++)
+    std::clog << (it!=_archive.begin() ? "," : "") << (*it).second.size();
+  std::clog << ")]\n";
 }
 
-void EvolvotronMain::History::goodbye(const MutatableImageDisplay* display)
+void EvolvotronMain::History::goodbye(MutatableImageDisplay* display)
 {
-  // First pass to delete any individual items 
-  for (
-       std::deque<std::pair<std::string,std::vector<std::pair<MutatableImageDisplay*,MutatableImage*> > > >::iterator it_out=_history.begin();
-       it_out!=_history.end();
-       it_out++
-       )
-    {
-      std::vector<std::pair<MutatableImageDisplay*,MutatableImage*> >::iterator it_in=(*it_out).second.begin();
-      while (it_in!=(*it_out).second.end())
-	{
-	  if ((*it_in).first==display)
-	    {
-	      delete (*it_in).second;
-	      it_in=(*it_out).second.erase(it_in);
-	    }
-	  else
-	    {
-	      it_in++;
-	    }
-	}
-    }
+  // First pass to delete any individual items for that display.
+  for (Archive::iterator it=_archive.begin();it!=_archive.end();it++)
+    (*it).second.erase(display);
 
   // Second pass to delete any undo items which are now empty
-  std::deque<std::pair<std::string,std::vector<std::pair<MutatableImageDisplay*,MutatableImage*> > > >::iterator it=_history.begin();
-  while (it!=_history.end())
+  Archive::iterator it=_archive.begin();
+  while (it!=_archive.end())
     {
-      if ((*it).second.empty())
-	{
-	  it=_history.erase(it);
-	}
-      else
-	{
-	  it++;
-	}
+      if ((*it).second.empty()) it=_archive.erase(it);
+      it++;
     }
 
   // Set menu label again in case we've changed the topmost item
-  const std::string action_name(_history.empty() ? "" : _history.front().first);
+  const std::string action_name(_archive.empty() ? "" : _archive.front().first);
   _main->set_undoable(undoable(),action_name);
 }
 
 void EvolvotronMain::History::replacing(MutatableImageDisplay* display)
 {
-  if (_history.size()==0)
+  if (_archive.size()==0)
     {
       begin_action("");
     }
   
-  MutatableImage*const image=display->image();
+  const boost::shared_ptr<const MutatableImage> image=display->image();
 
-  if (image!=0)
+  if (image.get())
     {
-      MutatableImage*const saved_image=image->deepclone();  // Deepclone doesn't copy locked state
-      saved_image->locked(image->locked());
-      _history.front().second.push_back(std::pair<MutatableImageDisplay*,MutatableImage*>(display,saved_image));
+      const boost::shared_ptr<const MutatableImage> saved_image(image->deepclone(image->locked()));
+      _archive.front().second.insert(std::make_pair(display,saved_image));
     }
 }
 
@@ -140,16 +102,16 @@ void EvolvotronMain::History::replacing(MutatableImageDisplay* display)
  */
 void EvolvotronMain::History::begin_action(const std::string& action_name)
 {
-  if (_history.size()==0 || _history.front().second.size()!=0)
+  if (_archive.size()==0 || _archive.front().second.size()!=0)
     {
-      _history.push_front(std::pair<std::string,std::vector<std::pair<MutatableImageDisplay*,MutatableImage*> > >());
+      _archive.push_front(ArchiveRecord());
 
-      assert(_history.front().second.size()==0);
+      assert(_archive.front().second.size()==0);
     }
   
-  _history.front().first=action_name;
+  _archive.front().first=action_name;
 
-  while (_history.size()>max_slots)
+  while (_archive.size()>max_slots)
     {
       purge();
     }
@@ -157,19 +119,21 @@ void EvolvotronMain::History::begin_action(const std::string& action_name)
 
 void EvolvotronMain::History::end_action()
 {
-  const std::string action_name(_history.empty() ? "" : _history.front().first);
+  const std::string action_name(_archive.empty() ? "" : _archive.front().first);
   _main->set_undoable(undoable(),action_name);
+
+  log_status();
 }
 
 bool EvolvotronMain::History::undoable()
 {
-  if (_history.size()==0)
+  if (_archive.size()==0)
     {
       return false;
     }
-  else if (_history.front().second.size()==0)
+  else if (_archive.front().second.size()==0)
     {
-      _history.pop_front();
+      _archive.pop_front();
       return undoable();
     }
   else
@@ -180,51 +144,47 @@ bool EvolvotronMain::History::undoable()
 
 void EvolvotronMain::History::undo()
 {
-  if (_history.size()==0)
+  if (_archive.size()==0)
     {
       // Shouldn't ever see this if Undo menu item is correctly greyed out.
       QMessageBox::warning(_main,"Evolvotron","Sorry, cannot undo any further");
     }
-  else if (_history.front().second.size()==0)
+  else if (_archive.front().second.size()==0)
     {
-      _history.pop_front();
+      _archive.pop_front();
       undo();
     }
   else
     {
-      for (
-	   std::vector<std::pair<MutatableImageDisplay*,MutatableImage*> >::iterator it=_history.front().second.begin();
-	   it!=_history.front().second.end();
+      for (ArchiveRecordEntries::iterator it=_archive.front().second.begin();
+	   it!=_archive.front().second.end();
 	   it++
 	   )
 	{
 	  _main->restore((*it).first,(*it).second);
 	}
-      _history.pop_front();
+      _archive.pop_front();
     }
 
-  const std::string action_name(_history.empty() ? "" : _history.front().first);
+  const std::string action_name(_archive.empty() ? "" : _archive.front().first);
   _main->set_undoable(undoable(),action_name);
 }
 
-void EvolvotronMain::last_spawned_image(const MutatableImage* image,SpawnMemberFn method)
+void EvolvotronMain::last_spawned_image(const boost::shared_ptr<const MutatableImage>& image,SpawnMemberFn method)
 {
-  delete _last_spawned_image;
-  _last_spawned_image=(image==0 ? 0 : image->deepclone());
+  _last_spawned_image=image;
   _last_spawn_method=method;
 }
 
 /*! Constructor sets up GUI components and fires up QTimer.
   Initialises mutation parameters using time, so different every time.
  */
-EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frames,uint framerate,uint n_threads,bool start_fullscreen,bool start_menuhidden)
+EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frames,uint framerate,uint n_threads,int niceness,bool start_fullscreen,bool start_menuhidden,bool function_debug_mode)
   :QMainWindow(parent,0,Qt::WType_TopLevel|Qt::WDestructiveClose)
-   ,_history(this)
-   ,_mutation_parameters(time(0),this)
+   ,_history(new EvolvotronMain::History(this))
+   ,_mutation_parameters(time(0),function_debug_mode,this)
    ,_statusbar_tasks(0)
-   ,_last_spawned_image(0)
    ,_last_spawn_method(&EvolvotronMain::spawn_normal)
-   ,_transform_factory(0)
 {
   setMinimumSize(600,400);
 
@@ -236,7 +196,7 @@ EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frame
   _statusbar_tasks_label=new QLabel("Ready",_statusbar);
   _statusbar->addWidget(_statusbar_tasks_label,0,true);
 
-  _dialog_about=new DialogAbout(this);
+  _dialog_about=new DialogAbout(this,n_threads);
   _dialog_help_short=new DialogHelp(this,false);
   _dialog_help_long=new DialogHelp(this,true);
 
@@ -248,7 +208,6 @@ EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frame
 
   _menubar=new QMenuBar(this);
 
-
   _popupmenu_file=new QPopupMenu;
   _popupmenu_file->insertItem("Re&set (Reset mutation parameters, clear locks)",this,SLOT(reset_cold()));
   _popupmenu_file->insertItem("&Restart (Preserve mutation parameters and locks)",this,SLOT(reset_warm()));
@@ -269,9 +228,9 @@ EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frame
   // We want to use a checkmark on some items
   _popupmenu_settings->setCheckable(true);
 
-  _popupmenu_settings->insertItem("&Mutations...",_dialog_mutation_parameters,SLOT(show()));
-  _popupmenu_settings->insertItem("&Functions...",_dialog_functions,SLOT(show()));
-  _popupmenu_settings->insertItem("&Favourite...",_dialog_favourite,SLOT(show()));
+  _popupmenu_settings->insertItem("&Mutation parameters...",_dialog_mutation_parameters,SLOT(show()));
+  _popupmenu_settings->insertItem("&Function weightings...",_dialog_functions,SLOT(show()));
+  _popupmenu_settings->insertItem("Fa&vourite function...",_dialog_favourite,SLOT(show()));
 
   _popupmenu_settings->insertSeparator();
 
@@ -327,7 +286,7 @@ EvolvotronMain::EvolvotronMain(QWidget* parent,const QSize& grid_size,uint frame
 	  this, SLOT(tick()) 
 	  );
 
-  _farm=new MutatableImageComputerFarm(n_threads);
+  _farm=std::auto_ptr<MutatableImageComputerFarm>(new MutatableImageComputerFarm(n_threads,niceness));
 
   //! \todo frames and framerate should be retained and modifiable from the GUI
   for (int r=0;r<grid_size.height();r++)
@@ -358,14 +317,29 @@ EvolvotronMain::~EvolvotronMain()
 {
   std::clog << "Evolvotron shut down begun...\n";
 
-  // Orphan any displays which outlived us (look out: shutdown order is Qt-determined)
+  // Orphan any displays which outlived us (and clear their images)  (look out: shutdown order is Qt-determined)
   for (std::set<MutatableImageDisplay*>::const_iterator it=_known_displays.begin();it!=_known_displays.end();it++)
     {
+      (*it)->image(boost::shared_ptr<const MutatableImage>());
       (*it)->main(0);
     }
 
+  std::clog << "...cleared displays, deleting farm...\n";
+
   // Shut down the compute farm
-  delete _farm;
+  _farm.reset();;
+
+  std::clog << "...deleted farm, deleting history...\n";
+
+  // Clean up records.
+  _last_spawned_image.reset();
+  _history.reset();
+
+  std::clog << "...deleted history\n";
+
+#ifndef NDEBUG
+  assert(InstanceCounted::is_clear());
+#endif
 
   std::clog << "...completed Evolvotron shutdown\n";  
 }
@@ -380,95 +354,46 @@ void EvolvotronMain::favourite_function_unwrapped(bool v)
   _dialog_favourite->favourite_function_unwrapped(v);
 }
 
-void EvolvotronMain::spawn_normal(const MutatableImage* image,MutatableImageDisplay* display)
+void EvolvotronMain::spawn_normal(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display)
 {
-  MutatableImage* new_image;
-  bool new_image_is_constant;
+  boost::shared_ptr<const MutatableImage> new_image;
 
   do
     {
       new_image=image->mutated(mutation_parameters());
-
-      new_image_is_constant=new_image->is_constant();
-
-      if (new_image_is_constant)
-	{
-	  delete new_image;
-	}
     }
-  while (new_image_is_constant);
-
+  while (new_image->is_constant());
+  
   history().replacing(display);
   display->image(new_image);
 }
 
-void EvolvotronMain::spawn_recoloured(const MutatableImage* image,MutatableImageDisplay* display)
+void EvolvotronMain::spawn_recoloured(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display)
 {  
+  std::auto_ptr<FunctionTop> new_root(image->top().typed_deepclone());
+  
+  new_root->reset_posttransform_parameters(mutation_parameters());
   history().replacing(display);
-
-  std::vector<FunctionNode*> args;
-  args.push_back(image->root()->deepclone());
-
-  //! \todo Option to use a FunctionPostTransformQuadratic might be interesting. How about a FunctionPostTransform*Generalised ?
-  display->image
-    (
-     new MutatableImage
-     (
-      new FunctionPostTransform
-      (
-       FunctionNode::stubparams(mutation_parameters(),12),
-       args,
-       0
-       ),
-      image->sinusoidal_z(),
-      image->spheremap()
-      )
-     );
+  boost::shared_ptr<const MutatableImage> it(new MutatableImage(new_root,image->sinusoidal_z(),image->spheremap(),false));
+  display->image(it);
 }
 
-void EvolvotronMain::spawn_warped(const MutatableImage* image,MutatableImageDisplay* display)
+void EvolvotronMain::spawn_warped(const boost::shared_ptr<const MutatableImage>& image,MutatableImageDisplay* display)
 {
+  std::auto_ptr<FunctionTop> new_root=std::auto_ptr<FunctionTop>(image->top().typed_deepclone());
+
   // Get the transform from whatever factory is currently set
   const Transform transform(transform_factory()(mutation_parameters().rng01()));
-
-  FunctionNode* new_root;
-  if (image->root()->is_a_FunctionPreTransform())
-    {
-      // If the image root is a transform wrapper then we can just concatentate transforms.
-      new_root=image->root()->deepclone();
-
-      FunctionPreTransform*const new_root_as_transform=new_root->is_a_FunctionPreTransform();
-      assert(new_root_as_transform!=0);
-
-      // Have to access params() through this to avoid protected scope of non-const version creating an error.
-      const FunctionPreTransform*const const_new_root_as_transform=new_root_as_transform;
-
-      Transform current_transform(const_new_root_as_transform->params());
-      current_transform.concatenate_on_right(transform);
-      new_root_as_transform->params(current_transform.get_columns());
-    }
-  else
-    {
-      // Otherwise have to create a new wrapper for the transform
-      std::vector<FunctionNode*> args;
-      args.push_back(image->root()->deepclone());
-      new_root=new FunctionPreTransform(transform.get_columns(),args,0);
-    }
-  
+      
+  new_root->concatenate_pretransform_on_right(transform);  
   history().replacing(display);
-  display->image(new MutatableImage(new_root,image->sinusoidal_z(),image->spheremap()));
+  boost::shared_ptr<const MutatableImage> it(new MutatableImage(new_root,image->sinusoidal_z(),image->spheremap(),false));
+  display->image(it);
 }
 
-void EvolvotronMain::restore(MutatableImageDisplay* display,MutatableImage* image)
+void EvolvotronMain::restore(MutatableImageDisplay* display,const boost::shared_ptr<const MutatableImage>& image)
 {
-  if (is_known(display))
-    {
-      display->image(image);
-    }
-  else
-    {
-      delete image;
-    }
+  if (is_known(display)) display->image(image);
 }
 
 void EvolvotronMain::set_undoable(bool v,const std::string& action_name)
@@ -509,7 +434,7 @@ void EvolvotronMain::spawn_all(MutatableImageDisplay* spawning_display,SpawnMemb
 
   // Issue new images (except to locked displays and to originator)
   // This will cause them to abort any running tasks
-  const MutatableImage*const spawning_image=spawning_display->image();
+  const boost::shared_ptr<const MutatableImage> spawning_image(spawning_display->image());
 
   last_spawned_image(spawning_image,method);
   
@@ -570,7 +495,7 @@ void EvolvotronMain::hello(MutatableImageDisplay* disp)
 
 void EvolvotronMain::goodbye(MutatableImageDisplay* disp)
 {
-  _history.goodbye(disp);
+  _history->goodbye(disp);
   _known_displays.erase(disp);  
 }
 
@@ -592,7 +517,7 @@ void EvolvotronMain::list_known(std::ostream& out) const
  */
 void EvolvotronMain::tick()
 {
-  const uint tasks=farm()->tasks();
+  const uint tasks=farm().tasks();
   if (tasks!=_statusbar_tasks)
     {
       if (tasks==0)
@@ -604,16 +529,16 @@ void EvolvotronMain::tick()
       _statusbar_tasks=tasks;
     }
 
-  MutatableImageComputerTask* task;
+  boost::shared_ptr<MutatableImageComputerTask> task;
 
   // If there are aborted jobs in the todo queue 
   // shift them straight over to done queue so the compute threads don't have to worry about them.
-  farm()->fasttrack_aborted();
+  farm().fasttrack_aborted();
 
   QTime watchdog;
   watchdog.start();
 
-  while ((task=farm()->pop_done())!=0)
+  while ((task=farm().pop_done())!=0)
     {
       if (is_known(task->display()))
 	{
@@ -623,7 +548,7 @@ void EvolvotronMain::tick()
 	{
 	  // If we don't know who owns it we just have to trash it 
 	  // (probably a top level window which was closed with incomplete tasks).
-	  delete task;
+	  task.reset();
 	}
 
       // Timeout in case we're being swamped by incoming tasks (maintain app responsiveness).
@@ -710,24 +635,28 @@ void EvolvotronMain::toggle_hide_menu()
  */
 void EvolvotronMain::reset(MutatableImageDisplay* display)
 {
-  FunctionNode* root;
+  std::auto_ptr<FunctionTop> root;
   if (_dialog_favourite->favourite_function().empty())
     {
-      root=FunctionNode::initial(mutation_parameters());
-    }
-  else if (_dialog_favourite->favourite_function_unwrapped())
-    {
-      FunctionNodeStubNewFnPtr rootfn=FunctionRegistry::get()->lookup(_dialog_favourite->favourite_function())->stubnew_fn();
-      root=(*rootfn)(mutation_parameters(),true);
+      root=std::auto_ptr<FunctionTop>(FunctionTop::initial(mutation_parameters()));
     }
   else
     {
-      root=FunctionNode::initial(mutation_parameters(),FunctionRegistry::get()->lookup(_dialog_favourite->favourite_function()));
+      root=std::auto_ptr<FunctionTop>
+	(
+	 FunctionTop::initial
+	 (
+	  mutation_parameters(),
+	  mutation_parameters().function_registry().lookup(_dialog_favourite->favourite_function()),
+	  _dialog_favourite->favourite_function_unwrapped()
+	  )
+	 );
     }
 
   history().replacing(display);
   //! \todo sinz and spheremap should be obtained from mutation parameters
-  display->image(new MutatableImage(root,!Args::global().option("-linz"),Args::global().option("-spheremap")));
+  const boost::shared_ptr<const MutatableImage> image(new MutatableImage(root,!Args::global().option("-linz"),Args::global().option("-spheremap"),false));
+  display->image(image);
 }
 
 void EvolvotronMain::undo()
@@ -755,6 +684,14 @@ void EvolvotronMain::reset(bool reset_mutation_parameters,bool clear_locks)
 {
   history().begin_action("reset/restart");
 
+  if (reset_mutation_parameters)
+    {
+      // Invoking reset on the 1st dialog actually resets the parameters 
+      _dialog_mutation_parameters->reset();
+      // This one just serves to setup the dialog from the now reset parameters
+      _dialog_functions->setup_from_mutation_parameters();
+    }
+
   for (std::vector<MutatableImageDisplay*>::iterator it=displays().begin();it!=displays().end();it++)
     {
       if (clear_locks)
@@ -764,15 +701,7 @@ void EvolvotronMain::reset(bool reset_mutation_parameters,bool clear_locks)
 	reset(*it);
     }
 
-  if (reset_mutation_parameters)
-    {
-      // Invoking reset on the 1st dialog actually resets the parameters 
-      _dialog_mutation_parameters->reset();
-      // This one just serves to setup the dialog from the now reset parameters
-      _dialog_functions->setup_from_mutation_parameters();
-    }
-
-  last_spawned_image(0,&EvolvotronMain::spawn_normal);
+  last_spawned_image(boost::shared_ptr<const MutatableImage>(),&EvolvotronMain::spawn_normal);
 
   history().end_action();
 }
