@@ -1,5 +1,5 @@
 // Source file for evolvotron
-// Copyright (C) 2002,2003 Tim Day
+// Copyright (C) 2007 Tim Day
 /*
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,24 +21,21 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   \brief Implementation of class MutatableImageDisplay.
 */
 
-#include <iostream>
-#include <fstream>
+#include "libevolvotron_precompiled.h"
+
+#include "mutatable_image_display.h"
 
-#include <qstring.h>
-#include <qimage.h>
-#include <qcursor.h>
-#include <qapplication.h>
 #include <qscrollview.h>
 #include <qmessagebox.h>
 #include <qfiledialog.h>
 #include <qpngio.h>
 
-#include "mutatable_image_display.h"
 #include "mutatable_image_display_big.h"
 #include "evolvotron_main.h"
 #include "mutatable_image_computer_task.h"
 #include "transform_factory.h"
 #include "function_pre_transform.h"
+#include "function_top.h"
 
 /*! The constructor is passed:
     - the owning widget (probably either a QGrid or null if top-level),
@@ -61,7 +58,7 @@ MutatableImageDisplay::MutatableImageDisplay(QWidget* parent,EvolvotronMain* mn,
    ,_timer(0)
    ,_resize_in_progress(false)
    ,_current_display_level(0)
-   ,_image(0)
+   ,_icon_serial(0LL)
    ,_properties(0)
    ,_menu(0)
    ,_menu_big(0)
@@ -167,28 +164,20 @@ MutatableImageDisplay::MutatableImageDisplay(QWidget* parent,EvolvotronMain* mn,
  */
 MutatableImageDisplay::~MutatableImageDisplay()
 {
-  assert(_image->ok());
+  assert(!_image.get() || _image->ok());
 
   // During app shutdown EvolvotronMain might have already been destroyed so only call it if it's there.
   // Don't use main() because it asserts non-null.
   if (_main)
     {
-      main()->farm()->abort_for(this);
+      main()->farm().abort_for(this);
       main()->goodbye(this);
     }
 
-  delete _image;
-  for (uint f=0;f<_offscreen_buffer.size();f++)
-    {
-      delete _offscreen_buffer[f];
-    }
+  _image.reset();
+  _offscreen_buffer.clear();
 
-  for (uint f=0;f<_offscreen_image.size();f++)
-    {
-      delete _offscreen_image[f];
-    }
-
-  std::clog << "A MutatableImageDisplay was deleted\n";
+  _offscreen_image.clear();
 }
 
 const uint MutatableImageDisplay::simplify_constants(bool single_action)
@@ -203,6 +192,7 @@ const uint MutatableImageDisplay::simplify_constants(bool single_action)
   _image->get_stats(old_nodes,old_parameters,old_depth,old_width,old_const);
 
   main()->history().replacing(this);
+
   image(_image->simplified());
 
   uint new_nodes;
@@ -218,6 +208,8 @@ const uint MutatableImageDisplay::simplify_constants(bool single_action)
 
   if (single_action)
     {
+      if (_icon.get()) _main->setIcon(*_icon);
+
       std::stringstream msg;
       msg << "Eliminated " << nodes_eliminated << " redundant function nodes\n";
       QMessageBox::information(this,"Evolvotron",msg.str().c_str(),QMessageBox::Ok);
@@ -249,29 +241,29 @@ void MutatableImageDisplay::frame_advance()
   repaint(false);
 }
 
-void MutatableImageDisplay::image(MutatableImage* i)
+void MutatableImageDisplay::image(const boost::shared_ptr<const MutatableImage>& i)
 {
-  assert(_image==0 || _image->ok());
-  assert(i==0 || i->ok());
+  assert(_image.get()==0 || _image->ok());
+  assert(i.get()==0 || i->ok());
 
-  // New image, so increment serial number so any old incoming stuff which avoids abort is ignored.
+  // New image, so increment serial number so any old incoming stuff which somehow avoids abort is ignored.
   _serial++;
 
   _properties->set_message(std::string("Not yet implemented"));
 
   // This might have already been done (e.g by resizeEvent), but it can't hurt to be sure.
-  main()->farm()->abort_for(this);
+  main()->farm().abort_for(this);
 
-  // Careful: we could be passed the same image again 
+  // Careful: we could be passed our own existing (and already owned) image
   // (a trick used by resize to trigger recompute & redisplay)
-  if (i!=_image)
+  // but if the image isn't being changed we might as well leave what's already shown
+  if (i.get()==0 || _image.get()==0 || i->serial()!=_image->serial())
     {
-      delete _image;
       _image=i;
 
       // Clear any existing image data - stops old animations continuing to play 
       for (uint f=0;f<_offscreen_buffer.size();f++)
-	_offscreen_buffer[f]->fill(black);
+	_offscreen_buffer[f].fill(black);
 
       // Queue a paint so the display will be blacked out
       repaint();
@@ -281,9 +273,9 @@ void MutatableImageDisplay::image(MutatableImage* i)
   _current_display_level=(uint)(-1);
 
   // Update lock status displayed in menu
-  _menu->setItemChecked(_menu_item_number_lock,(_image ? _image->locked() : false));
+  _menu->setItemChecked(_menu_item_number_lock,(_image.get() ? _image->locked() : false));
   
-  if (i!=0)
+  if (_image.get())
     {
       // Allow for displays up to 4096 pixels high or wide
       for (int level=12;level>=0;level--)
@@ -292,26 +284,19 @@ void MutatableImageDisplay::image(MutatableImage* i)
 	  
 	  if (image_size().width()>=s && image_size().height()>=s)
 	    {
-	      /*! \todo The image tree has no pixel or task specific state, so deepcloning is way overkill.
-		Would just need some referencing counting wrapper
-		(mainly to take ensure images aren't deleted while still being used by tasks) 
-		to avoid having to deepclone.
-		On the other hand this seems to work very robustly for now and isn't a killer.
-		*/
-	      const MutatableImage*const task_image=_image->deepclone();
-	      assert(task_image->ok());
-
 	      /*! \todo Should computed frames be constant or reduced c.f spatial resolution ?  (Do full z resolution for now)
 	       */
-	      MutatableImageComputerTask* task=new MutatableImageComputerTask(this,task_image,image_size()/s,_frames,level,_serial);
+	      const boost::shared_ptr<const MutatableImage> task_image(_image);
+	      assert(task_image->ok());
 
-	      main()->farm()->push_todo(task);
+	      const boost::shared_ptr<MutatableImageComputerTask> task(new MutatableImageComputerTask(this,task_image,image_size()/s,_frames,level,_serial));
+	      main()->farm().push_todo(task);
 	    }
 	}
     }
 }
 
-void MutatableImageDisplay::deliver(MutatableImageComputerTask* task)
+void MutatableImageDisplay::deliver(const boost::shared_ptr<const MutatableImageComputerTask>& task)
 {
   // Ignore tasks which were aborted or which have somehow got out of order 
   // (not impossible with multiple compute threads).
@@ -320,17 +305,9 @@ void MutatableImageDisplay::deliver(MutatableImageComputerTask* task)
    */
   if (!task->aborted() && task->level()<_current_display_level && task->serial()==_serial)
     {
-      for (uint f=0;f<_offscreen_image.size();f++)
-	{
-	  delete _offscreen_image[f];
-	}
       _offscreen_image.clear();
 
-      // Copy image data out of task 
-      // (do this by swapping to avoid a copy; the task is about to be deleted anyway)
-
-      _offscreen_image_data.clear();
-      _offscreen_image_data.swap(task->image_data());
+      _offscreen_image_data=task->image_data();
       
       for (uint f=0;f<task->frames();f++)
 	{
@@ -350,7 +327,20 @@ void MutatableImageDisplay::deliver(MutatableImageComputerTask* task)
 	  
 	  //! \todo Pick a scaling mode: smooth or not (or put it under GUI control). 
 	  // Curiously, although smoothscale seems to be noticeably slower, it doesn't look any better.
-	  _offscreen_buffer[f]->convertFromImage(_offscreen_image.back()->scale(image_size()));
+	  _offscreen_buffer[f].convertFromImage(_offscreen_image.back().scale(image_size()));
+	}
+
+      // For an icon, take the first image big enough to (hopefully) be filtered down nicely.
+      // The converter seems to auto-create an alpha mask sometimes (images with const-color areas), which is quite cool.
+      const QSize icon_size(32,32);
+      if (task->serial()!=_icon_serial && (task->level()==0 || (task->size().width()>=2*icon_size.width() && task->size().height()>=2*icon_size.height())))
+	{
+	  const QImage icon_image(_offscreen_image[_offscreen_image.size()/2].smoothScale(icon_size));
+	  
+	  _icon=std::auto_ptr<QPixmap>(new QPixmap(icon_size));
+	  _icon->convertFromImage(icon_image,QPixmap::Color);
+
+	  _icon_serial=task->serial();
 	}
 	  
       //! Note the resolution we've displayed so out-of-order low resolution images are dropped
@@ -360,22 +350,20 @@ void MutatableImageDisplay::deliver(MutatableImageComputerTask* task)
       //! \todo Any case for calling update() instead of repaint() ?  Repaint maybe feels smoother.
       repaint();
     }
-  
-  // This will also delete the task's deepcloned image
-  delete task;
-}
+  }
 
 void MutatableImageDisplay::lock(bool l,bool record_in_history)
 {
   // This might be called (with l=false) with null _image during start-up reset.
-  if (_image)
+  if (_image && _image->locked()!=l)
     {
       if (record_in_history)
 	{
 	  main()->history().begin_action(l ? "lock" : "unlock");
 	  main()->history().replacing(this);
 	}
-      _image->locked(l);
+      const boost::shared_ptr<const MutatableImage> new_image(_image->deepclone(l));
+      image(new_image);
       if (record_in_history)
 	{
 	  main()->history().end_action();
@@ -389,7 +377,7 @@ void MutatableImageDisplay::paintEvent(QPaintEvent*)
 {
   // Repaint the screen from the offscreen buffer
   // (If there have been resizes it will be black)
-  bitBlt(this,0,0,_offscreen_buffer[_current_frame]);
+  bitBlt(this,0,0,&_offscreen_buffer[_current_frame]);
 
   // If this is the first paint event after a resize we can start computing images for the new size.
   if (_resize_in_progress)
@@ -412,13 +400,13 @@ void MutatableImageDisplay::resizeEvent(QResizeEvent* event)
       _image_size=event->size();
       
       // Abort all current tasks because they'll be the wrong size.
-      main()->farm()->abort_for(this);
+      main()->farm().abort_for(this);
       
       // Resize and reset our offscreen buffer (something to do while we wait)
       for (uint f=0;f<_offscreen_buffer.size();f++)
 	{
-	  _offscreen_buffer[f]->resize(image_size());
-	  _offscreen_buffer[f]->fill(black);
+	  _offscreen_buffer[f].resize(image_size());
+	  _offscreen_buffer[f].fill(black);
 	}
       
       // Flag for the next paintEvent to tell it a recompute can be started now.
@@ -444,6 +432,8 @@ void MutatableImageDisplay::mousePressEvent(QMouseEvent* event)
     }
   else if (_full_functionality && event->button()==LeftButton)
     {
+      if (_icon.get()) _main->setIcon(*_icon);
+
       menupick_spawn();
     }
 }
@@ -461,7 +451,7 @@ void MutatableImageDisplay::mouseMoveEvent(QMouseEvent* event)
 	  const QPoint pixel_delta(event->pos()-_mid_button_adjust_last_pos);
 	  
 	  // Build the transform caused by the adjustment
-	  Transform transform;
+	  Transform transform=TransformIdentity();
 	  
 	  // Shift button (no ctrl) is various zooms
 	  if (event->state()&ShiftButton && !(event->state()&ControlButton))
@@ -570,40 +560,13 @@ void MutatableImageDisplay::mouseMoveEvent(QMouseEvent* event)
 	      std::clog << "[Pan]";
 	    }
 	  
-	  FunctionNode* new_root;
-	  if (image()->root()->is_a_FunctionPreTransform())
-	    {
-	      // If the image root is a transform wrapper then we can just concatentate transforms.
-	      new_root=image()->root()->deepclone();
-	      
-	      FunctionPreTransform*const new_root_as_transform=new_root->is_a_FunctionPreTransform();
-	      assert(new_root_as_transform!=0);
-	      
-	      // Have to access params() through this to avoid protected scope of non-const version creating an error.
-	      const FunctionPreTransform*const const_new_root_as_transform=new_root_as_transform;
-	      
-	      Transform current_transform(const_new_root_as_transform->params());
-	      //std::clog << "[Was: " << current_transform;
-	      //std::clog << ", concatenate: " << transform;
-	      current_transform.concatenate_on_right(transform);
-	      //std::clog << ", now: " << current_transform << "]";
-	      new_root_as_transform->params(current_transform.get_columns());
-	    }
-	  else
-	    {
-	      // Otherwise have to create a new wrapper for the transform:
-	      
-	      std::vector<real> params=transform.get_columns();
-	      
-	      std::vector<FunctionNode*> args;	  
-	      args.push_back(image()->root()->deepclone());
-	      
-	      new_root=new FunctionPreTransform(params,args,0);
-	    }
+	  std::auto_ptr<FunctionTop> new_root(image()->top().typed_deepclone());
+	  new_root->concatenate_pretransform_on_right(transform);
 
 	  // Install new image (triggers recompute).
-	  image(new MutatableImage(new_root,image()->sinusoidal_z(),image()->spheremap()));
-	  
+	  const boost::shared_ptr<const MutatableImage> new_image(new MutatableImage(new_root,image()->sinusoidal_z(),image()->spheremap(),false));
+	  image(new_image);
+
 	  // Finally, record position of this event as last event
 	  _mid_button_adjust_last_pos=event->pos();
 	}
@@ -707,6 +670,8 @@ void MutatableImageDisplay::menupick_simplify()
  */
 void MutatableImageDisplay::menupick_save_image()
 {
+  if (_icon.get()) _main->setIcon(*_icon);
+
   std::clog << "Save requested...\n";
 
   if (_current_display_level!=0)
@@ -743,21 +708,21 @@ void MutatableImageDisplay::menupick_save_image()
 	  
 	  if (!save_filename.isEmpty())
 	    {
-	      QFile* mng_file=0;
+	      std::auto_ptr<QFile> mng_file;
 	      if (save_format=="QT-MNG")
 		{
-		  mng_file=new QFile(save_filename);
+		  mng_file=std::auto_ptr<QFile>(new QFile(save_filename));
 		  if (!mng_file->open(IO_WriteOnly|IO_Truncate))
 		    {
 		      QMessageBox::critical(this,"Evolvotron","Failed to open file "+save_filename);
 		    }
 		  else 
 		    {
-		      QPNGImagePacker packer(mng_file,32,0);
+		      QPNGImagePacker packer(mng_file.get(),32,0);
 		      
 		      for (uint f=0;f<_offscreen_image.size();f++)
 			{
-			  if (!packer.packImage(*(_offscreen_image[f])))
+			  if (!packer.packImage(_offscreen_image[f]))
 			    {
 			      QMessageBox::critical(this,"Evolvotron","Failed while writing file "+save_filename+"\nFile will be removed");
 			      mng_file->close();
@@ -779,7 +744,6 @@ void MutatableImageDisplay::menupick_save_image()
 			      QMessageBox::critical(this,"Evolvotron","Failed to remove file "+save_filename);
 			    }
 			}
-		      delete mng_file;
 		    }
 		}
 	      else
@@ -803,7 +767,7 @@ void MutatableImageDisplay::menupick_save_image()
 			    }
 			}
 		      
-		      if (!_offscreen_image[f]->save(actual_save_filename.local8Bit(),save_format))
+		      if (!_offscreen_image[f].save(actual_save_filename.local8Bit(),save_format))
 			{
 			  QMessageBox::critical(this,"Evolvotron","Failed to write file "+actual_save_filename);
 			  if (f<_offscreen_image.size()-1)
@@ -822,6 +786,8 @@ void MutatableImageDisplay::menupick_save_image()
 
 void MutatableImageDisplay::menupick_save_function()
 {
+  if (_icon.get()) _main->setIcon(*_icon);
+
   QString save_filename=QFileDialog::getSaveFileName(".","Functions (*.xml)",this,"Save function","Save image function to an XML file");
   if (!save_filename.isEmpty())
     {
@@ -842,8 +808,8 @@ void MutatableImageDisplay::menupick_load_function()
     {
       std::ifstream file(load_filename.local8Bit());
       std::string report;
-      MutatableImage*const new_image=MutatableImage::load_function(file,report);
-      if (new_image==0)
+      boost::shared_ptr<const MutatableImage> new_image(MutatableImage::load_function(_main->mutation_parameters().function_registry(),file,report));
+      if (new_image.get()==0)
 	{
 	  QMessageBox::critical(this,"Evolvotron",("Function not loaded due to errors:\n"+report).c_str());
 	}
@@ -934,6 +900,8 @@ void MutatableImageDisplay::menupick_properties()
   msg << "Width           \t" << width << "\n";
   msg << "Constant        \t" << 100.0*proportion_constant << "%\n";
 
+  if (_icon.get()) _properties->setIcon(*_icon);
+
   _properties->set_message(msg.str());
   _properties->exec();
 }
@@ -944,6 +912,7 @@ void MutatableImageDisplay::menupick_properties()
 void MutatableImageDisplay::spawn_big(bool scrollable,const QSize& sz)
 {
   MutatableImageDisplayBig*const top_level_widget=new MutatableImageDisplayBig(0,main());
+  if (_icon.get()) top_level_widget->setIcon(*_icon);
 
   MutatableImageDisplay* display=0;
   
@@ -966,5 +935,5 @@ void MutatableImageDisplay::spawn_big(bool scrollable,const QSize& sz)
   if (main()->isFullScreen()) top_level_widget->showFullScreen();
 
   // Fire up image calculation
-  display->image(_image->deepclone());
+  display->image(_image);
 }
