@@ -23,10 +23,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef _mutatable_image_display_big_h_
 #define _mutatable_image_display_big_h_
 
-#include <qwidget.h>
-
-#include "useful.h"
-
 class EvolvotronMain;
 
 //! Intended to be used as a top-level widget holding a single MutatableImageDisplay
